unit Syn_main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Buttons, ExtCtrls;

type
  TMainForm = class(TForm)
    Memo: TMemo;
    Exit_B: TButton;
    Clear_B: TButton;
    Go_B: TButton;
    GroupBox1: TGroupBox;
    Count: TEdit;
    RCount: TEdit;
    Label12: TLabel;
    Label13: TLabel;
    StatusBar: TStatusBar;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    Fosc_E: TEdit;
    Fp_min_E: TEdit;
    Fp_max_E: TEdit;
    Fvco_min_E: TEdit;
    Fvco_max_E: TEdit;
    CB128: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure Fosc_EKeyPress(Sender: TObject; var Key: Char);
    procedure Fosc_EExit(Sender: TObject);
    procedure Fp_min_EKeyPress(Sender: TObject; var Key: Char);
    procedure Fp_min_EExit(Sender: TObject);
    procedure Fp_max_EKeyPress(Sender: TObject; var Key: Char);
    procedure Fp_max_EExit(Sender: TObject);
    procedure Fvco_min_EKeyPress(Sender: TObject; var Key: Char);
    procedure Fvco_min_EExit(Sender: TObject);
    procedure Fvco_max_EKeyPress(Sender: TObject; var Key: Char);
    procedure Fvco_max_EExit(Sender: TObject);
    procedure Exit_BClick(Sender: TObject);
    procedure Clear_BClick(Sender: TObject);
    procedure Go_BClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure Label1Click(Sender: TObject);
    procedure Label3Click(Sender: TObject);
    procedure Label5Click(Sender: TObject);
    procedure Label7Click(Sender: TObject);
    procedure Label9Click(Sender: TObject);
    procedure Label11Click(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    procedure DisableControls;
    procedure EnableControls;
  public
    { Public declarations }
  end;

  TCalcThread = class(TThread)
  private
    Line: String;
    P,V : Integer;
  protected
    procedure Execute; override;
    procedure InitC;
    procedure UpdM;
    procedure UpdC;
    procedure UpdR;
    procedure Cancel;
    procedure Complete;
  end;

var
  MainForm: TMainForm;
  CalcThr: TCalcThread;

implementation
{$R *.DFM}
const
  Foscmin=1000.0;
  Foscmax=20000.0;
  Fpmin=0.1;
  Fpmax=1000.0;
  Fvcomin=10000;
  Fvcomax=1100000.0;

var
  Fosc, Fp_min, Fp_max, Fvco_min, Fvco_max:Double;

procedure TMainForm.DisableControls;
begin
 Panel1.Enabled:=false;
 Go_B.Enabled:=false;
 Clear_B.Caption:='&Cancel';
 Clear_B.Hint:='Cancel calculation';
 StatusBar.SimpleText:='Press Esc to cancel';
end;

procedure TMainForm.EnableControls;
begin
 Panel1.Enabled:=true;
 Go_B.Enabled:=true;
 Clear_B.Caption:='&Clear';
 Clear_B.Hint:='Clear table';
 StatusBar.SimpleText:='';
end;

procedure TCalcThread.InitC;
begin
 MainForm.DisableControls;
 MainForm.Memo.Lines.BeginUpdate;
 MainForm.Memo.Lines.Clear;
 MainForm.Memo.Lines.EndUpdate;
 if MainForm.CB128.Checked then P:=128 else P:=64;
end;

procedure TCalcThread.Cancel;
begin
 MainForm.Memo.Lines.BeginUpdate;
 Line:='CALCULATION CANCELED';
 MainForm.Memo.Lines.Add(Line);
 MainForm.Memo.Lines.EndUpdate;
 MainForm.EnableControls;
end;

procedure TCalcThread.Complete;
begin
 MainForm.Memo.Lines.BeginUpdate;
 Line:='CALCULATION COMPLETE.';
 MainForm.Memo.Lines.Add(Line);
 Line:='wubblick@yahoo.com';
 MainForm.Memo.Lines.Add(Line);
 MainForm.Memo.Lines.EndUpdate;
 MainForm.EnableControls;
end;

procedure TCalcThread.UpdM;
begin
 MainForm.Memo.Lines.BeginUpdate;
 MainForm.Memo.Lines.Add(Line);
 MainForm.Memo.Lines.EndUpdate;
end;

procedure TCalcThread.UpdR;
begin
 MainForm.RCount.Text:=IntToStr(V);
 MainForm.RCount.Refresh;
end;

procedure TCalcThread.UpdC;
begin
 MainForm.Count.Text:=IntToStr(V);
 MainForm.Count.Refresh;
end;

procedure TCalcThread.Execute;
var
 R,Rmin,Rmax,N,A,MRATIO,Nmin,Nmax,Mmax: Integer;
 Temp,Fvco,Fp: Double;
begin
 Synchronize(InitC);
 if Fp_min>Fp_max then
  begin
   Temp:=Fp_max; Fp_max:=Fp_min; Fp_min:=Temp;
  end;
 if Fvco_min>Fvco_max then
  begin
   Temp:=Fvco_max; Fvco_max:=Fvco_min; Fvco_min:=Temp;
  end;
 Rmin:=round(Fosc/Fp_max);
 Rmax:=round(Fosc/Fp_min);
 Nmin:=round(((Fvco_min/Fp_max)/P)-1);
 Nmax:=round(((Fvco_max/Fp_min)/P)+1);
 MRATIO:=0; Mmax:=(P*Nmax)-1;
 for R:=Rmin to Rmax do
  begin
  V:=Rmax-R;
  Synchronize(UpdR);
  for N:=Nmin to Nmax do
   begin
   V:=Mmax-MRATIO;
   Synchronize(UpdC);
   for A:=0 to P-1 do
    begin
     if terminated then
      begin
       Synchronize(Cancel);
       exit;
      end;
     MRATIO:=(P*N)+A;
     Fp:=Fosc/R;
     Fvco:=Fp*MRATIO;
     if (Fvco>=Fvco_min) and (Fvco<=Fvco_max) then
      begin
       Line:=format('Fvco=%7.3f  Fp=%7.3f  DRATIO=%d'+
                    '  MRATIO=%d  P=%d  N=%d  A=%d',
                    [Fvco,Fp,R,MRATIO,P,N,A]);
       if (P=64) and (MRATIO>16383) then Line:=Line+'  - P must be 128';
       if (P=128) and (MRATIO<16384) then Line:=Line+'  - P must be 64';

       Synchronize(UpdM);
       if (R<8) or (R>16383) then
        begin
         Line:='***** Error: R out of range! *****';
         Synchronize(UpdM);
        end;
       if (N<16) or (N>2047) then
        begin
         Line:='***** Error: N out of range! *****';
         Synchronize(UpdM);
        end;
       if not (N>A) then
        begin
         Line:='***** Error: A out of range! *****';
         Synchronize(UpdM);
        end;
      end;
    end;
   end;
  end;
 Synchronize(Complete);
end;

function FloatConvert(E:TEdit; var V:Double; Min,Max:Double):Boolean;
var MV:Double; Msg:String;
begin
 MV:=V; Result:=true;
 try
 begin
  V := StrToFloat(E.Text);
  if (V<Min) or (V>Max) then raise EConvertError.Create('');
 end;
 except
 on EConvertError do
  begin
   Msg:=Format('Value must be between %.1f and %.0f KHz',[Min,Max]);
   MessageBeep(MB_IconError);
   MessageDlg(Msg,mtError,[mbOK],0);
   V:=MV; E.Text:=FloatToStr(V);
   E.SetFocus; E.SelectAll;
   Result:=false;
  end;
 end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
 Fosc:=12000.0;
 Fp_min:=12.5; Fp_max:=12.5;
 Fvco_min:=174000.0; Fvco_max:=174000.0;
 Fosc_E.Text:=FloatToStr(Fosc);
 Fp_min_E.Text:=FloatToStr(Fp_min);
 Fp_max_E.Text:=FloatToStr(Fp_max);
 Fvco_min_E.Text:=FloatToStr(Fvco_min);
 Fvco_max_E.Text:=FloatToStr(Fvco_max);
 KeyPreview:=true;
end;

procedure TMainForm.Exit_BClick(Sender: TObject);
begin
 Close;
end;

procedure TMainForm.Label1Click(Sender: TObject);
begin
 Fosc_E.SetFocus;
 Fosc_E.SelectAll;
end;

procedure TMainForm.Fosc_EKeyPress(Sender: TObject; var Key: Char);
begin
 if Key=#13 then
  begin
   FloatConvert(Fosc_E,Fosc,Foscmin,Foscmax);
   Key:=#0;
   Fosc_E.SelectAll;
  end;
end;

procedure TMainForm.Fosc_EExit(Sender: TObject);
begin
   FloatConvert(Fosc_E,Fosc,Foscmin,Foscmax);
end;

procedure TMainForm.Label3Click(Sender: TObject);
begin
 Fp_min_E.SetFocus;
 Fp_min_E.SelectAll;
end;

procedure TMainForm.Fp_min_EKeyPress(Sender: TObject; var Key: Char);
begin
 if Key=#13 then
  begin
   FloatConvert(Fp_min_E,Fp_min,Fpmin,Fpmax);
   FloatConvert(Fp_max_E,Fp_max,Fpmin,Fpmax);
   Key:=#0;
   Fp_min_E.SelectAll;
  end;
end;

procedure TMainForm.Fp_min_EExit(Sender: TObject);
begin
   FloatConvert(Fp_min_E,Fp_min,Fpmin,Fpmax);
   FloatConvert(Fp_max_E,Fp_max,Fpmin,Fpmax);
end;

procedure TMainForm.Label5Click(Sender: TObject);
begin
 Fp_max_E.SetFocus;
 Fp_max_E.SelectAll;
end;

procedure TMainForm.Fp_max_EKeyPress(Sender: TObject; var Key: Char);
begin
 if Key=#13 then
  begin
   FloatConvert(Fp_min_E,Fp_min,Fpmin,Fpmax);
   FloatConvert(Fp_max_E,Fp_max,Fpmin,Fpmax);
   Key:=#0;
   Fp_max_E.SelectAll;
  end;
end;

procedure TMainForm.Fp_max_EExit(Sender: TObject);
begin
   FloatConvert(Fp_min_E,Fp_min,Fpmin,Fpmax);
   FloatConvert(Fp_max_E,Fp_max,Fpmin,Fpmax);
end;

procedure TMainForm.Label7Click(Sender: TObject);
begin
 Fvco_min_E.SetFocus;
 Fvco_min_E.SelectAll;
end;

procedure TMainForm.Fvco_min_EKeyPress(Sender: TObject; var Key: Char);
begin
 if Key=#13 then
  begin
   FloatConvert(Fvco_min_E,Fvco_min,Fvcomin,Fvcomax);
   FloatConvert(Fvco_max_E,Fvco_max,Fvcomin,Fvcomax);
   Key:=#0;
   Fvco_min_E.SelectAll;
  end;
end;

procedure TMainForm.Fvco_min_EExit(Sender: TObject);
begin
   FloatConvert(Fvco_min_E,Fvco_min,Fvcomin,Fvcomax);
   FloatConvert(Fvco_max_E,Fvco_max,Fvcomin,Fvcomax);
end;

procedure TMainForm.Label9Click(Sender: TObject);
begin
 Fvco_max_E.SetFocus;
 Fvco_max_E.SelectAll;
end;

procedure TMainForm.Fvco_max_EKeyPress(Sender: TObject; var Key: Char);
begin
 if Key=#13 then
  begin
   FloatConvert(Fvco_min_E,Fvco_min,Fvcomin,Fvcomax);
   FloatConvert(Fvco_max_E,Fvco_max,Fvcomin,Fvcomax);
   Key:=#0;
   Fvco_max_E.SelectAll;
  end;
end;

procedure TMainForm.Fvco_max_EExit(Sender: TObject);
begin
   FloatConvert(Fvco_min_E,Fvco_min,Fvcomin,Fvcomax);
   FloatConvert(Fvco_max_E,Fvco_max,Fvcomin,Fvcomax);
end;

procedure TMainForm.Clear_BClick(Sender: TObject);
begin
 if (CalcThr<>nil) and (CalcThr.Terminated) then
   begin
    Memo.Lines.BeginUpdate;
    Memo.Lines.Clear;
    Memo.Lines.EndUpdate;
   end;
 if CalcThr<>nil then CalcThr.Terminate;
end;

procedure TMainForm.Go_BClick(Sender: TObject);
begin
 CalcThr:=TCalcThread.Create(False);
 CalcThr.FreeOnTerminate:=true;
end;

procedure TMainForm.SpeedButton1Click(Sender: TObject);
begin
 Fp_max_E.Text:=Fp_min_E.Text;
 FloatConvert(Fp_min_E,Fp_min,Fpmin,Fpmax);
 FloatConvert(Fp_max_E,Fp_max,Fpmin,Fpmax);
end;

procedure TMainForm.SpeedButton2Click(Sender: TObject);
begin
 Fvco_max_E.Text:=Fvco_min_E.Text;
 FloatConvert(Fvco_min_E,Fvco_min,Fvcomin,Fvcomax);
 FloatConvert(Fvco_max_E,Fvco_max,Fvcomin,Fvcomax);
end;

procedure TMainForm.Label11Click(Sender: TObject);
begin
 CB128.SetFocus;
 CB128.Checked:=not CB128.Checked;
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if Key=VK_ESCAPE then
 if CalcThr<>nil then CalcThr.Terminate;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
 if CalcThr<>nil then CalcThr.Terminate;
end;

end.
